<?php

/**
 * @file
 * General code for the Watchdog Notifier module.
 */


/**
 * Provides a wrapper on the list page to add a new entity.
 */
function watchdog_notifier_add() {
  // Create a basic entity structure to be used and passed to the validation
  // and submission functions.
  $entity = entity_get_controller('watchdog_notifier')->create();
  return drupal_get_form('watchdog_notifier_add_notifier_form', $entity);
}


/**
 * Cancel button for add/edit form
 */
function watchdog_notifier_cancel_button($form, &$form_state) {
  drupal_goto('admin/config/system/watchdog_notifier');
}


/**
 * Form deletion handler.
 */
function watchdog_notifier_do_delete($form, &$form_state) {
  $entity = $form_state['values']['watchdog_notifier_entity'];
  watchdog_notifier_delete($entity);
  drupal_set_message(t('The watchdog notifier @wn_id (@type (@severity) going to @email) has been deleted', array(
    '@wn_id'    => $entity->wn_id,
    '@type'     => $entity->type,
    '@severity' => watchdog_notifier_get_severity($entity->severity),
    '@email'    => $entity->email,
  )));
  $form_state['redirect'] = 'admin/config/system/watchdog_notifier';

  watchdog(
    'watchdog_notifier',
    'Notifier has been deleted: <pre>@config</pre>',
    array(
      '@config'  => print_r($entity, TRUE),
    ),
    WATCHDOG_INFO
  );
}


/**
 * Quick function to return the string equivalent of each of numerical severity
 * level.
 */
function watchdog_notifier_get_severity($notification_level = null) {
  $severity_array = array(
    -1 => 'All',
    0 => 'Emergency',
    1 => 'Alert',
    2 => 'Critical',
    3 => 'Error',
    4 => 'Warning',
    5 => 'Notice',
    6 => 'Informational',
    7 => 'Debug',
  );

  $severity_string = '';
  if ($notification_level == null) {
    $severity_string = 'No severity level has been provided.';
  } else {
    $severity_string = $severity_array[$notification_level];
  }

  return $severity_string;
}


/**
 * Callback for a page title when this entity is displayed.
 */
function watchdog_notifier_title($entity) {
  return t('@type (@severity) going to @email', array(
    '@type'     => $entity->type,
    '@severity' => watchdog_notifier_get_severity($entity->severity),
    '@email'    => $entity->email,
  ));
}


/**
 * autocomplete helper
 * $string = string for search
 */
function watchdog_notifier_types_autocomplete($string = NULL) {
  // Gather up all the watchdog log types.
  $type_array = array();
  $query = db_select('watchdog','w')
    ->fields('w', array('type'))
    ->distinct()
    ->orderBy('type', 'ASC');

  if (!empty($string)) {
    $query->condition('type', '%' . db_like($string) . '%', 'LIKE');
  }

  $result = $query->execute();
  foreach ($result as $res) {
    $type_array[check_plain($res->type)] = check_plain($res->type);
  }

  if (!empty($string)) {
    drupal_json_output($type_array);
  } else {
    return $type_array;
  }
}


/**
 * Implements the uri callback.
 */
function watchdog_notifier_uri($entity) {
  return array(
    'path' => 'admin/config/system/watchdog_notifier/view/' . $entity->wn_id,
  );
}

